#include "ParallelDomainScal.h"
#include "ProblemScal.h"
#include "ProblemInstat.h"

namespace AMDiS {

  ParallelDomainScal::ParallelDomainScal(const std::string& name,
					 ProblemScal *problem,
					 ProblemInstatScal *problemInstat)
    : ParallelDomainBase(name, 
			 problem, 
			 problemInstat, 
			 problem->getFESpace(),
			 problem->getRefinementManager()),
      probScal(problem)
  {
    info = problem->getInfo();
  }

  void ParallelDomainScal::initParallelization(AdaptInfo *adaptInfo)
  {
    FUNCNAME("ParallelDomainScal::initParallelization()");

    ParallelDomainBase::initParallelization(adaptInfo);

    DOFMatrix* m = probScal->getSystemMatrix();

    TEST_EXIT(m)("No DOF Matrix!\n");

    m->setIsRankDOF(isRankDof);
  }

  void ParallelDomainScal::solve()
  {
    FUNCNAME("ParallelDomainScal::solve()");

#ifdef _OPENMP
    double wtime = omp_get_wtime();
#endif
    clock_t first = clock();

    fillPetscMatrix(probScal->getSystemMatrix(), probScal->getRHS());      
    solvePetscMatrix(probScal->getSolution());

#ifdef _OPENMP
    INFO(info, 8)("solution of discrete system needed %.5f seconds system time / %.5f seconds wallclock time\n",
		   TIME_USED(first, clock()),
		   omp_get_wtime() - wtime);
#else
    INFO(info, 8)("solution of discrete system needed %.5f seconds\n",
		   TIME_USED(first, clock()));
#endif
  }

  Flag ParallelDomainScal::oneIteration(AdaptInfo *adaptInfo, Flag toDo)
  {
    FUNCNAME("ParallelDomainScal::oneIteration()");

    Flag flag =  dynamic_cast<StandardProblemIteration*>(iterationIF)->
      buildAndAdapt(adaptInfo, toDo);

    if (toDo.isSet(SOLVE))
      solve();

    if (toDo.isSet(SOLVE_RHS))
      ERROR_EXIT("Not yet implemented!\n");

    if (toDo.isSet(ESTIMATE))
      iterationIF->getProblem()->estimate(adaptInfo);

    return flag;
  }

}
